/****************************************************************************
    This program is protected under international and U.S. copyright laws
    as an unpublished work. This program is confidential and proprietary to
    Dolby Laboratories. Reproduction or disclosure, in whole or in part, or
    the production of derivative works therefrom without the express
    permission of Dolby Laboratories is prohibited.
    Copyright 2003-2011 by Dolby Laboratories. All rights reserved.

    Description: Dialogue Intelligence

    Date: $Date$

    Revision: $Revision$

***************************************************************************/
#ifndef _DI_H
#define _DI_H

#ifdef __cplusplus
extern "C" {
#endif

/* Includes */
#include <stddef.h>

#include "src.h"
#include "bst.h"
#include "dly.h"
#include "sfv.h"
#include "ast.h"
#include "psc.h"
#include "zcs.h"
#include "zcm.h"
#include "rpm.h"
#include "lrm.h"

/* Defines */

/* Version */
#define DI_VERSION              ( "1.0.0" )

/* SPEECH/OTHER id's */
#define INVALID                 -1
#define OTHER                   0
#define SPEECH                  1

/* Sample rate for feature extraction and boosting */
#define FEATURE_SAMPLE_RATE     16000

/* Number of features */
#define NUM_FEATURES            7

/* Overlap between successive feature frames (4 => 75% overlap) */
#define OVERLAP                 4

/* The frame size is defined below.  The block sizes, for each feature, are defined in the
 * corresponding feature header file (e.g. ast.h, lrm.h etc.).
 *
 * The frame and block sizes should not be changed.
 */

/* Boosting frame size, in samples */
#define FEATURE_FRAME_SIZE      32768

/* The maximum feature block size - must be the maximum of
 * the feature bock sizes: AST_BLOCK_SIZE, LRM_BLOCK_SIZE, PCM_BLOCK_SIZE,
 * RPM_BLOCK_SIZE, SFV_BLOCK_SIZE, ZCM_BLOCK_SIZE, ZCS_BLOCK_SIZE
 */
#define MAX_FEATURE_BLOCK_SIZE  1024

/* Number of "maximum sized" blocks per frame */
#define MAX_BLOCKS_PER_FRAME    ( (FEATURE_FRAME_SIZE / MAX_FEATURE_BLOCK_SIZE) / OVERLAP )

/* Feature vector indicies, one for each feature */
#define AST_IDX                 1
#define LRM_IDX                 6
#define PSC_IDX                 2
#define RPM_IDX                 5
#define SFV_IDX                 0
#define ZCM_IDX                 4
#define ZCS_IDX                 3

/* Instance structure */
typedef struct
{
    void    *p_ast;
    void    *p_lrm;
    void    *p_psc;
    void    *p_rpm;
    void    *p_sfv;
    void    *p_zcm;
    void    *p_zcs;
    void    *p_dly;
    void    *p_bst;
    void    *p_src;
    float   p_feature_vector[NUM_FEATURES];
    float   *p_src_in_data;
    float   *p_src_out_data;
    int     max_block_count;
    float   *p_scaled_samples;
    int     is_speech;
} DI;

/* Public prototypes */
size_t  di_query_mem_size(unsigned int sample_rate, unsigned int max_input_samples);
int     di_init(void *p_inst, size_t num_bytes, unsigned int sample_rate, unsigned int max_input_samples);
int     di_process(void *p_inst, float *p_samples, unsigned int num_samples);

#ifdef __cplusplus
}
#endif

#endif /* _DI_H */
