/****************************************************************************
    This program is protected under international and U.S. copyright laws
    as an unpublished work. This program is confidential and proprietary to
    Dolby Laboratories. Reproduction or disclosure, in whole or in part, or
    the production of derivative works therefrom without the express
    permission of Dolby Laboratories is prohibited.
    Copyright 2002-2011 by Dolby Laboratories. All rights reserved.

    Description: FFT routine

    Date: $Date$

    Revision: $Revision$

***************************************************************************/

#include "fft.h"

extern const float p_wreal[FFT_MAX_N_LOG2][FFT_MAX_N_LOG2];
extern const float p_wimag[FFT_MAX_N_LOG2][FFT_MAX_N_LOG2];

/*******************************************************************************************
int fftl(float *p_real, float *p_imag, unsigned int N)
-Purpose:
    Perform an N point FFT on complex inputs with outputs written in place.  This function
    provides outputs already bit reversed.  N must be a power of 2 between 2 and 1024.
-Parameters:
    float *p_real       - (in/out) Pointer to the real part of N input/output samples
    float *p_imag       - (in/out) Pointer to the imaginary part of N input/output samples
    unsigned int N      - (in) Size of the FFT.  Must be a power of 2 between 2 and 1024.
-Returns:
    0 on success, or -1 on error.
********************************************************************************************/
int fftl(float *p_real, float *p_imag, unsigned int N)
{
    unsigned int    m, le, ip, mp;
    unsigned int    i, j, k, l;
    float           treal, timag, ureal, uimag;
        
    switch (N)
    {
    case 1024:  m = 10;
                break;
    case 512:   m = 9;
                break;
    case 256:   m = 8;
                break;
    case 128:   m = 7;
                break;
    case 64:    m = 6;
                break;
    case 32:    m = 5;
                break;
    case 16:    m = 4;
                break;
    case 8:     m = 3;
                break;
    case 4:     m = 2;
                break;
    case 2:     m = 1;
                break;
    default:    /* Error! */
                return -1;
    }
    
    mp = ((unsigned int) FFT_MAX_N_LOG2) - m;
    
    for (l = 0; l < (m - 1); l++)
    {
        le = 1 << (m - l);
        ureal = 1;
        uimag = 0;
        
        for (j = 0; j < (le >> 1); j++)
        {
            for (i = j; i < N; i += le)
            {
                ip = i + (le >> 1);
                treal = p_real[i] + p_real[ip];
                timag = p_imag[i] + p_imag[ip];
                p_real[ip] = p_real[i] - p_real[ip];
                p_imag[ip] = p_imag[i] - p_imag[ip];
                p_real[i] = treal;
                p_imag[i] = timag;
                treal = p_real[ip];
                p_real[ip] = p_real[ip] * ureal - p_imag[ip] * uimag;
                p_imag[ip] = p_imag[ip] * ureal + treal * uimag;
            }
            treal = ureal;
            ureal = ureal * p_wreal[mp][l] - uimag * p_wimag[mp][l];
            uimag = uimag * p_wreal[mp][l] + treal * p_wimag[mp][l];
        }
    }
    
    for (i = 0; i < N; i += 2)
    {
        ip = i + 1;
        treal = p_real[i] + p_real[ip];
        timag = p_imag[i] + p_imag[ip];
        p_real[ip] = p_real[i] - p_real[ip];
        p_imag[ip] = p_imag[i] - p_imag[ip];
        p_real[i] = treal;
        p_imag[i] = timag;
    }
    
    j = 0;
    for (i = 0; i < (N - 1); i++)
    {
        if (i < j)
        {
            treal = p_real[j];
            timag = p_imag[j];
            p_real[j] = p_real[i];
            p_imag[j] = p_imag[i];
            p_real[i] = treal;
            p_imag[i] = timag;
        }
        k = (N >> 1);
        while (k <= j)
        {
            j = j - k;
            k = k >> 1;
        }
        j = j + k;
    }
    
    return 0;
}
